<?php

declare(strict_types=1);

namespace Drupal\search_gov_results_api\Form;

use Drupal\Component\Serialization\Json;
use Drupal\Component\Utility\UrlHelper;
use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Entity\EntityDisplayModeBase;
use Drupal\Core\Entity\EntityDisplayRepositoryInterface;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Form\FormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Messenger\MessengerInterface;
use Drupal\Core\Render\RendererInterface;
use Drupal\Core\Url;
use Drupal\node\Entity\Node;
use Drupal\path_alias\AliasManagerInterface;
use GuzzleHttp\Client;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Symfony\Component\HttpFoundation\RequestStack;

/**
 * Provides a Search.gov Results API form.
 */
final class SearchPageForm extends FormBase {

  /**
   * @var \GuzzleHttp\Client
   */
  protected Client $httpClient;

  private ?\Symfony\Component\HttpFoundation\Request $request;

  /**
   * @var true
   */
  protected bool $configurationErrors = FALSE;

  private array $results;

  /**
   * @var false|mixed
   */
  protected mixed $requestResponse;

  /**
   * @var \Drupal\Core\Entity\EntityDisplayModeBase
   */
  protected EntityDisplayModeBase $entityDisplayMode;

  /**
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  protected EntityTypeManagerInterface $entityTypeManager;

  protected \Drupal\Core\Entity\EntityViewBuilderInterface $nodeViewBuilder;

  /**
   * @var \Drupal\path_alias\AliasManagerInterface
   */
  protected AliasManagerInterface $aliasManager;

  /**
   * @var \Drupal\Core\Render\RendererInterface
   */
  protected RendererInterface $renderer;


  /**
   * @param \Drupal\Core\Session\AccountInterface $account
   */
  public function __construct(Client $httpClient, RequestStack $requestStack, ConfigFactoryInterface $configFactory, MessengerInterface $messenger, AliasManagerInterface $aliasManager, EntityTypeManagerInterface $entityTypeManager, RendererInterface $renderer) {
    $this->httpClient = $httpClient;
    $this->requestStack = $requestStack;
    $this->request = $this->requestStack->getCurrentRequest();
    $this->configFactory = $configFactory;
    $this->messenger = $messenger;
    $this->aliasManager = $aliasManager;
    $this->entityTypeManager = $entityTypeManager;
    $this->nodeViewBuilder = $this->entityTypeManager->getViewBuilder('node');
    $this->renderer = $renderer;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new self(
      $container->get('http_client'),
      $container->get('request_stack'),
      $container->get('config.factory'),
      $container->get('messenger'),
      $container->get('path_alias.manager'),
      $container->get('entity_type.manager'),
      $container->get('renderer')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function getFormId(): string {
    return 'search_gov_results_api_search_page';
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state): array {
    $form_state->setMethod('GET');
    $search_query = $_GET['search'] ?? $_POST['search'] ?? $this->request->get('search');

    $form['search'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Search Query'),
      '#default_value' => $search_query,
    ];

    // add search filters (facets)

    $form['actions'] = [
      '#type' => 'actions',
      'submit' => [
        '#type' => 'submit',
        '#value' => $this->t('Send'),
      ],
    ];

    // if there is a search query fetch the results
    if (!empty($search_query)) {
      $request = $this->generateApiRequest($search_query);
    }
    // TODO: check the cache for reseponse data based on this query

    if (isset($request) && !empty($request)) {
      $response = $this->fetchRequestResponse($request);
      $this->results = $response['web']['results'];
      $this->requestResponse = $response;
      $results_items = $this->processResults();
      // TODO: cache the response data
    }


    // add the results
    $form['results_list'] = [
      '#type' => 'container',
      '#attributes' => ['class' => ['node-list']],
      'items' => [
        '#theme' => 'item_list',
        '#items' => $results_items,
        '#type' => 'ul',
      ],
      '#weight' => 30,
    ];


    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function validateForm(array &$form, FormStateInterface $form_state): void {
    // @todo Validate the form here.
    // Example:
    // @code
    //   if (mb_strlen($form_state->getValue('message')) < 10) {
    //     $form_state->setErrorByName(
    //       'message',
    //       $this->t('Message should be at least 10 characters.'),
    //     );
    //   }
    // @endcode
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state): void {
    #$this->messenger()->addStatus($this->t('The message has been sent.'));
    #$values = $form_state->getValues();
    #$form_state->setRedirect('<front>');
  }

  /**
   * @param mixed $search_query
   *
   * @return string
   */
  protected function generateApiRequest(mixed $search_query) {
    $endpoint = 'https://api.gsa.gov/technology/searchgov/v2/results/i14y';
    $config = $this->configFactory->get('search_gov_results_api.settings');

    $parameters = [
      'affiliate' => $config->get('site_handle') ?? NULL,
      'access_key' => $config->get('api_key') ?? NULL,
      'query' => $search_query,
      'include_facets' => $config->get('facets')['enable'] ? 'true' : 'false',
    ];

    if (!isset($parameters['affiliate']) || empty($parameters['affiliate'])) {
      $this->configurationError('Site Handle');
      $this->configurationErrors = TRUE;
    }
    if (!isset($parameters['access_key']) || empty($parameters['access_key'])) {
      $this->configurationError('API Access Key');
      $this->configurationErrors = TRUE;
    }

    if ($this->configurationErrors) {
      return FALSE;
    }

    if ($config->get('highlight_results')) {
      $parameters['enable_highlighting'] = $config->get('highlight_results');
    }

    $parameters['limit'] = $config->get('results_per_page') ?? 20;

    $parameters['offset'] = ($_GET['page'] ?? 0 )* $parameters['limit'];

    // TODO
    // $parameters['sort_by']


    return $endpoint . '?' .  UrlHelper::buildQuery($parameters);

  }

  /**
   * Set a configuration error.
   *
   * @param $field
   *
   * @return void
   */
  protected function configurationError($field) {
    $this->setError('Cannot perform search, search module misconfiguration.', 'The Search.gov Results API module configuration has no data for the @field field. <a href=":href">Please update the configuration here.</a>', ['@field' => $field , ':href' => Url::fromRoute('search_gov_results_api.settings')->toString()]);

  }

  /**
   * Set error messages one for the user and optionally one for the admin (if
   * the current user is logged in)
   *
   * @param string $public_message
   * @param $admin_message
   * @param $args
   *
   * @return void
   */
  protected function setError(string $public_message, $admin_message = '', $args = []) {
    $this->messenger->addError($this->t($public_message, $args));
    if (!empty($admin_message) && $this->currentUser()->hasPermission('administer site configuration')) {
      // set the admin message error
      $this->messenger->addError($this->t($admin_message, $args));
    }
  }

  /**
   * Make the request and return the response
   *
   * @param string $request
   *
   * @return false|mixed
   * @throws \GuzzleHttp\Exception\GuzzleException
   */
  protected function fetchRequestResponse(string $request) {
    // TODO: add exception handling
    $response = $this->httpClient->get($request);
    $body = $response->getBody();
    $body_contents = $body->getContents();
    if (empty($body_contents)) {
      return FALSE;
    }
    $data = Json::decode($body_contents);

    return $data;
  }

  /**
   * Process the results (figure out the nid for all of the pages)
   *
   * @return void
   */
  protected function processResults() {$this->getResultNodes();
    $this->getResultNodes();
    $this->adjustSnippets();

    // assemble the render array
    $build = [];
    $renderedItems = [];
    foreach ($this->results as $result) {
      $build[] = $result['build'];
      $renderedItems[] = $this->renderer->render($result['build']);
    }



    // add the pager


    // return the render array
    return $build;
  }

  protected function getResultNodes() {
    $config = $this->configFactory->get('search_gov_results_api.settings');
    $display_mode = str_replace('node.', '', $config->get('display_mode'));
    $site_base = $config->get('site_base');

    // start by getting the NODE IDs
    $nids = [];
    foreach ($this->results AS $key => &$result) {
      $system_path = str_replace($site_base, '', $result['url']);
      $nids[$key] = $result['nid'] = $this->nidFromAlias($system_path);

    }

    if (empty($nids)) {
      return FALSE;
    }

    // make sure there are no empty entries
    foreach ($nids AS $key => $nid) {
      if (empty($nid)) {
        unset($nids[$key]);
      }
    }

    // load the nodes
    $nodes = Node::loadMultiple($nids);

    $build = $this->nodeViewBuilder->viewMultiple($nodes, $display_mode);

    // match the nodes up with the results
    foreach ($this->results AS &$result) {
      $result['node'] = $nodes[$result['nid']] ?? FALSE;
      $result['build'] = $build[$result['nid']] ?? FALSE;
    }

  }

  protected function nidFromAlias($path) {

    if (empty($path)) {
       return NULL;
    }

    $alias = $this->aliasManager->getPathByAlias($path);

    $matches = [];

    if (preg_match('/^\/node\/(\d+)$/', $alias, $matches)) {
      // The second match is the node ID.
      $nid =  $matches[1];
      return $nid;
    }

    return NULL;

  }

  /**
   * replace the unicode characters surrounding the search phrase with a span to
   * highlight the search terms in the snippet and the title (if it exists)
   *
   * @return void
   */
  private function adjustSnippets() {
    if (!isset($this->results) || empty($this->results)) {
      return;
    }
    foreach ($this->results AS &$result) {
      $replacements_tags = [
        '' => '<span class="highlight">',
        '' => '</span>',
      ];
      $replacements_empty = ['' => '', '' => ''];

      // replace the markers at the start and end of the
      $result['snippet_w_tags'] = strtr($result['snippet'], $replacements_tags);
      $result['snippet_clean'] = strtr($result['snippet'], $replacements_empty);

      $result['title_w_tags'] = strtr($result['title'], $replacements_tags);
      $result['clean_title'] = strtr($result['title'], $replacements_empty);
      #$result['build']['#title'] = $result['title'] = str_replace('', '</span>', $result['title']);
      if (!empty($result['nid']) && !empty($result['node'])) {
        // since there is a nid and node we can render as the node
        $result['build'] =[
          '#theme' => 'search_result',
          '#plugin_id' => 'search_gov_results_api',
          '#result' => [
            'link' => $result['url'], // absolute link to the page
            'type' => $result['node']->getType(), // content type
            'title' => $result['clean_title'], // node title
            'node' => $result['node'], // node
            'extra' => [],
            'score' => '',// the search query score
            'snippet' => [
              '#plain_text' => strip_tags($result['snippet_w_tags']),
            ],
          ],
        ];
      }
      else {
        // we don't have a nid or node so just render the results with the link
        // to the page
        $result['build'] = [
          '#type' => 'container',
          'link' => [
            '#theme' => 'link',
            '#title' => $result['clean_title'],
            '#url' => $result['url'],
          ],
          'text' => [
            '#type' => 'markup',
            '#markup' => $result['snippet_w_tags'],
          ]
        ];
      }


    }
    $true = TRUE;
  }

}
