<?php

declare(strict_types=1);

namespace Drupal\search_gov_results_api\Form;

use Drupal\Core\Entity\EntityDisplayModeBase;
use Drupal\Core\Form\ConfigFormBase;
use Drupal\Core\Form\FormStateInterface;

/**
 * Configure Search.gov Results API settings for this site.
 */
final class SettingsForm extends ConfigFormBase {

  /**
   * {@inheritdoc}
   */
  public function getFormId(): string {
    return 'search_gov_results_api_settings';
  }

  /**
   * {@inheritdoc}
   */
  protected function getEditableConfigNames(): array {
    return ['search_gov_results_api.settings'];
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state): array {
    $form['site_base'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Site Base URL'),
      '#description' => $this->t('This is the base url that Search.gov indexes. I.E.: https://dhs.gov'),
      '#default_value' => $this->config('search_gov_results_api.settings')->get('site_base'),
      '#required' => TRUE,
    ];
    $form['site_handle'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Site Handle'),
      '#description' => $this->t('The Site Handled give to your site on search.gov. This is to differentiate between your site and another in the search index.'),
      '#default_value' => $this->config('search_gov_results_api.settings')->get('site_handle'),
      '#required' => TRUE,
    ];
    $form['api_key'] = [
      '#type' => 'textfield',
      '#title' => $this->t('API Access Key'),
      '#description' => $this->t('the API key from search.gov'),
      '#default_value' => $this->config('search_gov_results_api.settings')->get('api_key'),
      '#required' => TRUE,
    ];
    $form['highlight_results'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Highlight Results'),
      '#description' => $this->t('When enabled the search query will be highlighted in the returned results.'),
      '#default_value' => $this->config('search_gov_results_api.settings')->get('highlight_results'),
    ];
    $form['results_per_page'] = [
      '#type' => 'select',
      '#title' => $this->t('Number of Results Per Page'),
      '#description' => $this->t('When search results are returned how many per page do you want listed?'),
      '#default_value' => $this->config('search_gov_results_api.settings')->get('results_per_page') ?? 20,
      '#options' => [
        5 => 5,
        10 => 10,
        20 => 20,
        50 => 50,
        100 => 100,
      ],
    ];

    $form['facets'] = [
      '#type' => 'fieldset',
      '#title' => $this->t('Search Filters (facets)'),
      '#tree' => TRUE,
    ];
    $form['facets']['enable'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Enable'),
      '#description' => $this->t('Enable search filters on the search results page.'),
      '#default_value' => $this->config('search_gov_results_api.settings')->get('facets')['enable'] ?? TRUE,
    ];
    $form['facets']['searchgov_custom1'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Search.gov Custom Field 1'),
      '#description' => $this->t('The name by which you want the search filter to be known on the search results page.'),
      '#default_value' => $this->config('search_gov_results_api.settings')->get('facets')['searchgov_custom1'] ?? '',
    ];
    $form['facets']['searchgov_custom2'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Search.gov Custom Field 2'),
      '#description' => $this->t('The name by which you want the search filter to be known on the search results page.'),
      '#default_value' => $this->config('search_gov_results_api.settings')->get('facets')['searchgov_custom2'] ?? '',
    ];
    $form['facets']['searchgov_custom3'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Search.gov Custom Field 3'),
      '#description' => $this->t('The name by which you want the search filter to be known on the search results page.'),
      '#default_value' => $this->config('search_gov_results_api.settings')->get('facets')['searchgov_custom3'] ?? '',
    ];
    $display_repository = \Drupal::service('entity_display.repository');
    $view_modes = $display_repository->getViewModes('node');
    $display_modes = [];
    foreach ($view_modes AS $mode) {
      $display_modes[$mode['id']] = $mode['label'];
    }
    $form['display_mode'] = [
      '#type' => 'select',
      '#title' => $this->t('Display Mode'),
      '#desciption' => $this->t('Which display mode would you like to use for the search results?'),
      '#options' => $display_modes,
      '#default_value' => $this->config('search_gov_results_api.settings')->get('display_mode') ?? 'node.search_result',
    ];
    return parent::buildForm($form, $form_state);
  }

  /**
   * {@inheritdoc}
   */
  public function validateForm(array &$form, FormStateInterface $form_state): void {
    // @todo Validate the form here.
    // Example:
    // @code
    //   if ($form_state->getValue('example') === 'wrong') {
    //     $form_state->setErrorByName(
    //       'message',
    //       $this->t('The value is not correct.'),
    //     );
    //   }
    // @endcode
    parent::validateForm($form, $form_state);
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state): void {
    $values = $form_state->cleanValues()->getValues();
    $config = $this->config('search_gov_results_api.settings');
    // loop through all values and add to the config
    foreach ($values AS $key => $value) {
      $config->set($key, $value);
    }
    $config->save();
    parent::submitForm($form, $form_state);
  }

}
